% Rearrange.m can provide the results of ANSYS along the interface accoding
% to the sequence of the elliptical parameter, psi.

% Before clicking the "Run" button, the following must be done.

% First, we need to import the information in 'Interface node.txt' by Matlab (HOME -> Import Data),
% and name it "NODE", NODE=[Node#, X, Y]; 

% Second, we need to import the information in 'Interface element.txt' by Matlab (HOME -> Import Data),
% and name them "ELE1", "ELE2". 
% (Separately paste the information in MATRIX and INCLUSION sides !!!)
% ELE1=[ELEM#, MAT, TYP, ESY, SEC, NODE1, NODE2, NODE3, NODE4]; % ELE1: elements in matrix.
% ELE2=[ELEM#, MAT, TYP, ESY, SEC, NODE1, NODE2, NODE3, NODE4]; % ELE1: elements in inclusion.

% Third, we need to import the information in 'Element solution.txt' by Matlab (HOME -> Import Data),
% and name it "ELE_sol" (element solutions). 
% ELE_sol=[ELEM#, N11, N22, N12, M11, M22, M12];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                    Import Procedure: 
% 1. Click the "Import Data" button (HOME -> Import Data)
% 2. Select the text file
% 3. Select "Numeric Matrix" in "IMPORTED DATA"
% 4. Select "Exclude rows with" & "unimportable cells" in "UNIMPORTABLE CELLS"
% 5. Rename the matrix name
% 6. Adjust the width of each column properly
% 7. Click the "Import Selection" button
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clc
a=0.2;  b=0.15;    % a,b: Size of elliptical interface.
Nif=size(NODE,1);   % Nif: Number of nodes along interface;
Psi=zeros(Nif,1);  NODE_re=zeros(size(NODE)); % Default;
% 're' means rearranged; 
for k=1:Nif
if NODE(k,3)>=0, Psi(k)=acosd(NODE(k,2)/a);
else Psi(k)=360-acosd(NODE(k,2)/a); end
end
[Psi_re,Id]=sort(Psi);
for k=1:Nif
    NODE_re(k,:)=NODE(Id(k),:);
end

if (max(ELEall(:,4))-min(ELEall(:,4))==0) && (max(ELEall(:,4))==1), ELEall(:,4:end)=ELEall(:,5:end); end
Mif1=0; Mif2=0;
for k=1:size(ELEall,1)
if ELEall(k,5)==1, Mif1=Mif1+1; ELE1(Mif1,:)=ELEall(k,:); end   % Mif1: Number of elements in matrix side along interface.
if ELEall(k,5)==2, Mif2=Mif2+1; ELE2(Mif2,:)=ELEall(k,:); end   % Mif2: Number of elements in inclusion side along interface.
end
max(ELE2)-min(ELE2)  % Check if some numerical values are wrongly cut during the import procedure.
max(ELE1)-min(ELE1)  % Check if some numerical values are wrongly cut during the import procedure.
if Mif1~=Mif2, disp('Dissimilar mesh on matrix and inclusion sides!!'); 
               disp('Please adjust the mesh in Ansys!'); end
           
% Rearrange the sequence of ELEM# by the elliptical parameter, psi.
ELE1_re=zeros(size(ELE1));  ELE2_re=zeros(size(ELE2));  % Default
for m=1:Nif, for n=1:Mif1
    if ELE1(n,8)==NODE_re(m,1), ELE1_re(m,:)=ELE1(n,:); end
end
end
for m=1:Nif, for n=1:Mif2
    if ELE2(n,7)==NODE_re(m,1), ELE2_re(m,:)=ELE2(n,:); end
end
end
err=0;
if max(ELE1_re(:,5))~=min(ELE1_re(:,5)), err=1; end
if max(ELE2_re(:,5))~=min(ELE2_re(:,5)), err=1; end
if err==1, disp('Some elements are not rearranged!!');
           disp('Please check manually from Workspace.'); end

ELE1_solre=zeros(Mif1,7);  ELE2_solre=zeros(Mif2,7);  % Default
for m=1:Nif
for n=1:(Mif1+Mif2)
if ELE_sol(n,1)==ELE1_re(m,1), ELE1_solre(m,:)=ELE_sol(n,:); end
if ELE_sol(n,1)==ELE2_re(m,1), ELE2_solre(m,:)=ELE_sol(n,:); end
end
end